@@ -2,27 +2,29 @@ package Test2::Hub::Subtest;
 use strict;
 use warnings;
 
-our $VERSION = '1.302059';
-
+our $VERSION = '1.302120';
 
 BEGIN { require Test2::Hub; our @ISA = qw(Test2::Hub) }
-use Test2::Util::HashBase qw/nested bailed_out exit_code manual_skip_all id/;
+use Test2::Util::HashBase qw/nested exit_code manual_skip_all/;
 use Test2::Util qw/get_tid/;
 
-my $ID = 1;
-sub init {
+sub is_subtest { 1 }
+
+sub inherit {
     my $self = shift;
-    $self->SUPER::init(@_);
-    $self->{+ID} ||= join "-", $$, get_tid, $ID++;
+    my ($from) = @_;
+
+    $self->SUPER::inherit($from);
+
+    $self->{+NESTED} = $from->nested + 1;
 }
 
-sub process {
-    my $self = shift;
-    my ($e) = @_;
-    $e->set_nested($self->nested);
-    $e->set_in_subtest($self->{+ID});
-    $self->set_bailed_out($e) if $e->isa('Test2::Event::Bail');
-    $self->SUPER::process($e);
+{
+    # Legacy
+    no warnings 'once';
+    *ID = \&Test2::Hub::HID;
+    *id = \&Test2::Hub::hid;
+    *set_id = \&Test2::Hub::set_hid;
 }
 
 sub send {
@@ -32,9 +34,15 @@ sub send {
     my $out = $self->SUPER::send($e);
 
     return $out if $self->{+MANUAL_SKIP_ALL};
-    return $out unless $e->isa('Test2::Event::Plan')
-        && $e->directive eq 'SKIP'
-        && ($e->trace->pid != $self->pid || $e->trace->tid != $self->tid);
+
+    my $f = $e->facet_data;
+
+    my $plan = $f->{plan} or return $out;
+    return $out unless $plan->{skip};
+
+    my $trace = $f->{trace} or die "Missing Trace!";
+    return $out unless $trace->{pid} != $self->pid
+                    || $trace->{tid} != $self->tid;
 
     no warnings 'exiting';
     last T2_SUBTEST_WRAPPER;
@@ -42,13 +50,18 @@ sub send {
 
 sub terminate {
     my $self = shift;
-    my ($code, $e) = @_;
+    my ($code, $e, $f) = @_;
     $self->set_exit_code($code);
 
     return if $self->{+MANUAL_SKIP_ALL};
-    return if $e->isa('Test2::Event::Plan')
-           && $e->directive eq 'SKIP'
-           && ($e->trace->pid != $$ || $e->trace->tid != get_tid);
+
+    $f ||= $e->facet_data;
+
+    if(my $plan = $f->{plan}) {
+        my $trace = $f->{trace} or die "Missing Trace!";
+        return if $plan->{skip}
+               && ($trace->{pid} != $$ || $trace->{tid} != get_tid);
+    }
 
     no warnings 'exiting';
     last T2_SUBTEST_WRAPPER;
@@ -113,7 +126,7 @@ F<http://github.com/Test-More/test-more/>.
 
 =head1 COPYRIGHT
 
-Copyright 2016 Chad Granum E<lt>exodist@cpan.orgE<gt>.
+Copyright 2017 Chad Granum E<lt>exodist@cpan.orgE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
