@@ -63,19 +63,18 @@ d_attribute_pure='undef'
 d_attribute_unused='undef'
 d_attribute_warn_unused_result='undef'
 d_backtrace='undef'
-d_bcmp='undef'
-d_bcopy='undef'
 d_bsd='undef'
 d_bsdgetpgrp='undef'
 d_bsdsetpgrp='undef'
+d_builtin_add_overflow='undef'
 d_builtin_choose_expr='undef'
 d_builtin_expect='undef'
-d_bzero='undef'
+d_builtin_mul_overflow='undef'
+d_builtin_sub_overflow='undef'
 d_c99_variadic_macros='undef'
 d_casti32='undef'
 d_castneg='undef'
 d_cbrt='undef'
-d_charvspr='undef'
 d_chown='undef'
 d_chroot='undef'
 d_chsize='undef'
@@ -287,6 +286,7 @@ d_llrint='undef'
 d_llrintl='undef'
 d_llround='undef'
 d_llroundl='undef'
+d_localeconv_l='undef'
 d_localtime64='undef'
 d_localtime_r='undef'
 d_localtime_r_needs_tzset='undef'
@@ -312,15 +312,12 @@ d_madvise='undef'
 d_malloc_good_size='undef'
 d_malloc_size='undef'
 d_mblen='undef'
+d_mbrlen='undef'
+d_mbrtowc='undef'
 d_mbstowcs='undef'
 d_mbtowc='undef'
-d_memchr='define'
-d_memcmp='define'
-d_memcpy='define'
 d_memmem='undef'
-d_memmove='undef'
 d_memrchr='undef'
-d_memset='define'
 d_mkdir='undef'
 d_mkdtemp='undef'
 d_mkfifo='undef'
@@ -348,6 +345,7 @@ d_msync='undef'
 d_munmap='undef'
 d_mymalloc='undef'
 d_nan='undef'
+d_nanosleep='undef'
 d_ndbm_h_uses_prototypes='undef'
 d_nearbyint='undef'
 d_newlocale='undef'
@@ -406,9 +404,6 @@ d_rewinddir='undef'
 d_rint='undef'
 d_rmdir='undef'
 d_round='undef'
-d_safebcpy='undef'
-d_safemcpy='undef'
-d_sanemcmp='undef'
 d_sbrkproto='undef'
 d_scalbn='undef'
 d_scalbnl='undef'
@@ -483,7 +478,6 @@ d_socket='undef'
 d_socklen_t='undef'
 d_sockpair='undef'
 d_socks5_init='undef'
-d_sprintf_returns_strlen='undef'
 d_sqrtl='undef'
 d_srand48_r='undef'
 d_srandom_r='undef'
@@ -504,17 +498,16 @@ d_stdiobase='undef'
 d_stdstdio='undef'
 d_strchr='undef'
 d_strcoll='undef'
-d_strctcpy='undef'
-d_strerrm='strerror(e)'
-d_strerror='undef'
 d_strerror_l='undef'
 d_strerror_r='undef'
 d_strftime='undef'
 d_strlcat='undef'
 d_strlcpy='undef'
+d_strnlen='undef'
 d_strtod='undef'
 d_strtol='define'
 d_strtold='undef'
+d_strtold_l='undef'
 d_strtoll='undef'
 d_strtoq='undef'
 d_strtoul='define'
@@ -534,6 +527,7 @@ d_tcsetpgrp='undef'
 d_telldir='undef'
 d_telldirproto='undef'
 d_tgamma='undef'
+d_thread_safe_nl_langinfo_l='undef'
 d_time='define'
 d_timegm='undef'
 d_times='undef'
@@ -564,8 +558,6 @@ d_vfork='undef'
 d_void_closedir='undef'
 d_voidsig='undef'
 d_voidtty=''
-d_volatile='undef'
-d_vprintf='define'
 d_vsnprintf='undef'
 d_wait4='undef'
 d_waitpid='undef'
@@ -643,7 +635,6 @@ i64type='int64_t'
 i8size='1'
 i8type='signed char'
 i_arpainet='undef'
-i_assert='define'
 i_bfd='undef'
 i_bsdioctl=''
 i_crypt='undef'
@@ -654,7 +645,6 @@ i_dlfcn='undef'
 i_execinfo='undef'
 i_fcntl='undef'
 i_fenv='undef'
-i_float='undef'
 i_fp='undef'
 i_fp_class='undef'
 i_gdbm='undef'
@@ -665,13 +655,10 @@ i_ieeefp='undef'
 i_inttypes='undef'
 i_langinfo='undef'
 i_libutil='undef'
-i_limits='undef'
 i_locale='undef'
 i_machcthr='undef'
 i_malloc='undef'
 i_mallocmalloc='undef'
-i_math='define'
-i_memory='undef'
 i_mntent='undef'
 i_ndbm='undef'
 i_netdb='undef'
@@ -687,12 +674,9 @@ i_rpcsvcdbm='undef'
 i_sgtty='undef'
 i_shadow='undef'
 i_socks='undef'
-i_stdarg='define'
 i_stdbool='undef'
-i_stddef='define'
 i_stdint='undef'
 i_stdlib='define'
-i_string='define'
 i_sunmath='undef'
 i_sysaccess='undef'
 i_sysdir='undef'
@@ -729,10 +713,8 @@ i_time='define'
 i_unistd='undef'
 i_ustat='undef'
 i_utime='undef'
-i_values='undef'
-i_varargs='undef'
-i_varhdr='stdarg.h'
 i_vfork='undef'
+i_wchar='undef'
 i_xlocale='undef'
 ignore_versioned_solibs='y'
 inc_version_list_init='NULL'
@@ -918,7 +900,6 @@ uvsize='4'
 uvtype='unsigned long'
 uvuformat='"lu"'
 uvxformat='"lx"'
-vaproto='undef'
 vendorarch=''
 vendorarchexp=''
 vendorlib_stem=''
