@@ -5,10 +5,9 @@ _o='.o'
 afs='false'
 afsroot='/afs'
 alignbytes='8'
-ansi2knr=''
 aphostname='/bin/hostname'
-archlib='/usr/local/lib/perl5/5.25/unknown'
-archlibexp='/usr/local/lib/perl5/5.25/unknown'
+archlib='/usr/local/lib/perl5/5.27/unknown'
+archlibexp='/usr/local/lib/perl5/5.27/unknown'
 archname='unknown64'
 asctime_r_proto='0'
 bin='/usr/local/bin'
@@ -64,19 +63,18 @@ d_attribute_pure='undef'
 d_attribute_unused='undef'
 d_attribute_warn_unused_result='undef'
 d_backtrace='undef'
-d_bcmp='undef'
-d_bcopy='undef'
 d_bsd='undef'
 d_bsdgetpgrp='undef'
 d_bsdsetpgrp='undef'
+d_builtin_add_overflow='undef'
 d_builtin_choose_expr='undef'
 d_builtin_expect='undef'
-d_bzero='undef'
+d_builtin_mul_overflow='undef'
+d_builtin_sub_overflow='undef'
 d_c99_variadic_macros='undef'
 d_casti32='undef'
 d_castneg='undef'
 d_cbrt='undef'
-d_charvspr='undef'
 d_chown='undef'
 d_chroot='undef'
 d_chsize='undef'
@@ -108,6 +106,14 @@ d_dlerror='undef'
 d_dlopen='undef'
 d_dlsymun='undef'
 d_dosuid='undef'
+d_double_has_inf='define'
+d_double_has_nan='define'
+d_double_has_negative_zero='define'
+d_double_has_subnormals='define'
+d_double_style_cray='undef'
+d_double_style_ibm='undef'
+d_double_style_ieee='define'
+d_double_style_vax='undef'
 d_drand48_r='undef'
 d_drand48proto='undef'
 d_dup2='undef'
@@ -133,6 +139,7 @@ d_expm1='undef'
 d_faststdio='undef'
 d_fchdir='undef'
 d_fchmod='undef'
+d_fchmodat='undef'
 d_fchown='undef'
 d_fcntl='undef'
 d_fcntl_can_lock='undef'
@@ -172,6 +179,7 @@ d_fsync='undef'
 d_ftello='undef'
 d_ftime='undef'
 d_futimes='undef'
+d_gai_strerror='undef'
 d_gdbm_ndbm_h_uses_prototypes='undef'
 d_gdbmndbm_h_uses_prototypes='undef'
 d_getaddrinfo='undef'
@@ -273,10 +281,12 @@ d_lgamma='undef'
 d_lgamma_r='undef'
 d_libm_lib_version='undef'
 d_link='undef'
+d_linkat='undef'
 d_llrint='undef'
 d_llrintl='undef'
 d_llround='undef'
 d_llroundl='undef'
+d_localeconv_l='undef'
 d_localtime64='undef'
 d_localtime_r='undef'
 d_localtime_r_needs_tzset='undef'
@@ -285,6 +295,11 @@ d_lockf='undef'
 d_log1p='undef'
 d_log2='undef'
 d_logb='undef'
+d_long_double_style_ieee='undef'
+d_long_double_style_ieee_doubledouble='undef'
+d_long_double_style_ieee_extended='undef'
+d_long_double_style_ieee_std='undef'
+d_long_double_style_vax='undef'
 d_longdbl='undef'
 d_longlong='undef'
 d_lrint='undef'
@@ -297,14 +312,12 @@ d_madvise='undef'
 d_malloc_good_size='undef'
 d_malloc_size='undef'
 d_mblen='undef'
+d_mbrlen='undef'
+d_mbrtowc='undef'
 d_mbstowcs='undef'
 d_mbtowc='undef'
-d_memchr='define'
-d_memcmp='define'
-d_memcpy='define'
 d_memmem='undef'
-d_memmove='undef'
-d_memset='define'
+d_memrchr='undef'
 d_mkdir='undef'
 d_mkdtemp='undef'
 d_mkfifo='undef'
@@ -332,6 +345,7 @@ d_msync='undef'
 d_munmap='undef'
 d_mymalloc='undef'
 d_nan='undef'
+d_nanosleep='undef'
 d_ndbm_h_uses_prototypes='undef'
 d_nearbyint='undef'
 d_newlocale='undef'
@@ -346,6 +360,7 @@ d_old_pthread_create_joinable='undef'
 d_oldpthreads='undef'
 d_oldsock='undef'
 d_open3='undef'
+d_openat='undef'
 d_pathconf='undef'
 d_pause='undef'
 d_perl_otherlibdirs='undef'
@@ -384,13 +399,11 @@ d_regcomp='undef'
 d_remainder='undef'
 d_remquo='undef'
 d_rename='define'
+d_renameat='undef'
 d_rewinddir='undef'
 d_rint='undef'
 d_rmdir='undef'
 d_round='undef'
-d_safebcpy='undef'
-d_safemcpy='undef'
-d_sanemcmp='undef'
 d_sbrkproto='undef'
 d_scalbn='undef'
 d_scalbnl='undef'
@@ -465,7 +478,6 @@ d_socket='undef'
 d_socklen_t='undef'
 d_sockpair='undef'
 d_socks5_init='undef'
-d_sprintf_returns_strlen='undef'
 d_sqrtl='undef'
 d_srand48_r='undef'
 d_srandom_r='undef'
@@ -486,17 +498,16 @@ d_stdiobase='undef'
 d_stdstdio='undef'
 d_strchr='undef'
 d_strcoll='undef'
-d_strctcpy='undef'
-d_strerrm='strerror(e)'
-d_strerror='undef'
 d_strerror_l='undef'
 d_strerror_r='undef'
 d_strftime='undef'
 d_strlcat='undef'
 d_strlcpy='undef'
+d_strnlen='undef'
 d_strtod='undef'
 d_strtol='define'
 d_strtold='undef'
+d_strtold_l='undef'
 d_strtoll='undef'
 d_strtoq='undef'
 d_strtoul='define'
@@ -516,6 +527,7 @@ d_tcsetpgrp='undef'
 d_telldir='undef'
 d_telldirproto='undef'
 d_tgamma='undef'
+d_thread_safe_nl_langinfo_l='undef'
 d_time='define'
 d_timegm='undef'
 d_times='undef'
@@ -532,6 +544,7 @@ d_ualarm='undef'
 d_umask='undef'
 d_uname='undef'
 d_union_semun='undef'
+d_unlinkat='undef'
 d_unordered='undef'
 d_unsetenv='undef'
 d_uselocale='undef'
@@ -545,8 +558,6 @@ d_vfork='undef'
 d_void_closedir='undef'
 d_voidsig='undef'
 d_voidtty=''
-d_volatile='undef'
-d_vprintf='define'
 d_vsnprintf='undef'
 d_wait4='undef'
 d_waitpid='undef'
@@ -561,6 +572,7 @@ db_prefixtype='size_t'
 db_version_major='0'
 db_version_minor='0'
 db_version_patch='0'
+default_inc_excludes_dot='undef'
 direntrytype='struct dirent'
 doubleinfbytes='0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x7f'
 doublekind='3'
@@ -623,7 +635,6 @@ i64type='long'
 i8size='1'
 i8type='signed char'
 i_arpainet='undef'
-i_assert='define'
 i_bfd='undef'
 i_bsdioctl=''
 i_crypt='undef'
@@ -634,7 +645,6 @@ i_dlfcn='undef'
 i_execinfo='undef'
 i_fcntl='undef'
 i_fenv='undef'
-i_float='undef'
 i_fp='undef'
 i_fp_class='undef'
 i_gdbm='undef'
@@ -645,13 +655,10 @@ i_ieeefp='undef'
 i_inttypes='undef'
 i_langinfo='undef'
 i_libutil='undef'
-i_limits='undef'
 i_locale='undef'
 i_machcthr='undef'
 i_malloc='undef'
 i_mallocmalloc='undef'
-i_math='define'
-i_memory='undef'
 i_mntent='undef'
 i_ndbm='undef'
 i_netdb='undef'
@@ -667,12 +674,9 @@ i_rpcsvcdbm='undef'
 i_sgtty='undef'
 i_shadow='undef'
 i_socks='undef'
-i_stdarg='define'
 i_stdbool='undef'
-i_stddef='define'
 i_stdint='undef'
 i_stdlib='define'
-i_string='define'
 i_sunmath='undef'
 i_sysaccess='undef'
 i_sysdir='undef'
@@ -709,10 +713,9 @@ i_time='define'
 i_unistd='undef'
 i_ustat='undef'
 i_utime='undef'
-i_values='undef'
-i_varargs='undef'
-i_varhdr='stdarg.h'
 i_vfork='undef'
+i_wchar='undef'
+i_xlocale='undef'
 ignore_versioned_solibs='y'
 inc_version_list_init='NULL'
 installstyle='lib/perl5'
@@ -767,8 +770,8 @@ package='perl5'
 perl_static_inline='static'
 phostname='hostname'
 pidtype=int
-privlib='/usr/local/lib/perl5/5.25'
-privlibexp='/usr/local/lib/perl5/5.25'
+privlib='/usr/local/lib/perl5/5.27'
+privlibexp='/usr/local/lib/perl5/5.27'
 procselfexe=''
 prototype='undef'
 ptrsize='8'
@@ -818,11 +821,11 @@ sig_name_init='0'
 sig_num_init='0'
 sig_size='1'
 signal_t=int
-sitearch='/usr/local/lib/perl5/5.25/unknown'
-sitearchexp='/usr/local/lib/perl5/5.25/unknown'
-sitelib='/usr/local/lib/perl5/5.25'
+sitearch='/usr/local/lib/perl5/5.27/unknown'
+sitearchexp='/usr/local/lib/perl5/5.27/unknown'
+sitelib='/usr/local/lib/perl5/5.27'
 sitelib_stem='/usr/local/lib/perl5'
-sitelibexp='/usr/local/lib/perl5/5.25'
+sitelibexp='/usr/local/lib/perl5/5.27'
 sizesize=4
 sizetype='size_t'
 socksizetype='int'
@@ -897,7 +900,6 @@ uvsize='8'
 uvtype='unsigned long'
 uvuformat='"lu"'
 uvxformat='"lx"'
-vaproto='undef'
 vendorarch=''
 vendorarchexp=''
 vendorlib_stem=''
