@@ -491,10 +491,10 @@ Perl_new_version(pTHX_ SV *ver)
 	    ver = SvRV(ver);
 
 	/* Begin copying all of the elements */
-	if ( hv_existss(MUTABLE_HV(ver), "qv") )
+	if ( hv_exists(MUTABLE_HV(ver), "qv", 2) )
 	    (void)hv_stores(MUTABLE_HV(hv), "qv", newSViv(1));
 
-	if ( hv_existss(MUTABLE_HV(ver), "alpha") )
+	if ( hv_exists(MUTABLE_HV(ver), "alpha", 5) )
 	    (void)hv_stores(MUTABLE_HV(hv), "alpha", newSViv(1));
 	{
 	    SV ** svp = hv_fetchs(MUTABLE_HV(ver), "width", FALSE);
@@ -840,7 +840,7 @@ Perl_vnumify(pTHX_ SV *vs)
 	Perl_croak(aTHX_ "Invalid version object");
 
     /* see if various flags exist */
-    if ( hv_existss(MUTABLE_HV(vs), "alpha") )
+    if ( hv_exists(MUTABLE_HV(vs), "alpha", 5 ) )
 	alpha = TRUE;
 
     if (alpha) {
@@ -978,7 +978,7 @@ Perl_vstringify(pTHX_ SV *vs)
 	    return &PL_sv_undef;
     }
     else {
-	if ( hv_existss(MUTABLE_HV(vs), "qv") )
+	if ( hv_exists(MUTABLE_HV(vs), "qv", 2) )
 	    return VNORMAL(vs);
 	else
 	    return VNUMIFY(vs);
